@@ -22,6 +22,7 @@ from .system.constants import *
 from .role_manager.constants import *
 from .unique_brain.constants import *
 from .packet import *
+from .transport import Transport
 
 from .util import now, log, logv
 from .pack import PackTuple, PackType, jdpack, jdunpack
@@ -179,21 +180,17 @@ def _service_matches(dev: 'Device', serv: bytearray):
             return False
     return True
 
-
-class Transport:
-    """A base class for packet transports"""
-
-    on_receive: Optional[Callable[[bytes], None]] = None
-    # Callback to report a received packet to the bus
-
-    def send(self, pkt: bytes) -> None:
-        # send a packet payload over the transport layer
-        pass
-
-
 def rand_u64():
     return bytearray([getrandbits(8) for _ in range(8)])
 
+def is_raspberrypi():
+    # https://raspberrypi.stackexchange.com/questions/5100/detect-that-a-python-program-is-running-on-the-pi
+    try:
+        import io
+        with io.open('/sys/firmware/devicetree/base/model', 'r') as m:
+            if 'raspberry pi' in m.read().lower(): return True
+    except Exception: pass
+    return False
 
 class Bus(EventEmitter):
     """A Jacdac bus that managed devices, service client, registers."""
@@ -209,7 +206,7 @@ class Bus(EventEmitter):
                  disable_settings: bool = False,
                  disable_brain: bool = False,
                  disable_dev_tools: bool = False,
-                 spi: bool = False,
+                 spi: Optional[bool] = None,
                  hf2_portname: Optional[str] = None,
                  transport_cmd: Optional[str] = None,
                  default_logger_min_priority: Optional[int] = None,
@@ -233,7 +230,7 @@ class Bus(EventEmitter):
             disable_dev_tools (bool, optional): Do not try to connect to developer tools server.
             hf2_portname (str, optional): port name exposing HF2 packets.
             transport_cmd (str, optional): name of executable to run as a transport.
-            spi (bool, optional): use SPI for transport.
+            spi (bool, optional): use SPI for transport. Enabled by default if Raspberry Pi and spi is None.
         """
         super().__init__(self)
 
@@ -274,12 +271,16 @@ class Bus(EventEmitter):
         self.storage_dir = storage_dir or cfg.get("storage_dir", "./.jacdac")
         self.hf2_portname = hf2_portname or cfg.get("hf2_portname")
         self.transport_cmd = transport_cmd or cfg.get("transport_cmd")
+        # automatically turn on SPI transport on the Pi
+        if spi is None:
+            spi = is_raspberrypi()
+        self.spi = spi or cfg.getboolean("spi", None)
 
         self.self_device = Device(self, device_id, bytearray(4))
         self.process_thread = threading.Thread(target=self._process_task)
         self.process_thread.daemon = True
         self.transports: List[Transport] = transports or []
-        if not disable_dev_tools:
+        if not self.disable_dev_tools:
             from .transports.ws import WebSocketTransport
             self.transports.append(WebSocketTransport(DEVTOOLS_SOCKET_URL))
         if self.transport_cmd:
@@ -288,7 +289,7 @@ class Bus(EventEmitter):
         if self.hf2_portname:
             from .transports.hf2 import HF2Transport
             self.transports.append(HF2Transport(self.hf2_portname))        
-        if spi:
+        if self.spi:
             from .transports.spi import SpiTransport
             self.transports.append(SpiTransport())
 
@@ -1689,7 +1690,7 @@ class Device(EventEmitter):
 
     @ property
     def is_connected(self):
-        return self.clients != None
+        return len(self.clients) != 0
 
     @ property
     def short_id(self):
@@ -1734,7 +1735,7 @@ class Device(EventEmitter):
         self.debug("destroy")
         for c in self.clients:
             c._detach()
-        self.clients = None  # type: ignore
+        self.clients = []
 
     def _log_report_prefix(self) -> str:
         return "{}>".format(self.short_id)
@@ -1783,3 +1784,56 @@ class Device(EventEmitter):
                 # log(`handle pkt at ${client.role} rep=${pkt.serviceCommand}`)
                 c.device = self
                 c.handle_packet_outer(pkt)
+
+class BufferClient(Client):
+    _value: bytearray
+    _dirty: bool
+
+    """
+    A client that handles a double-buffer bytes buffer
+    """
+    def __init__(self, bus: Bus, service_class: int, pack_formats: Dict[int, str], role: str) -> None:
+        super().__init__(bus, service_class, pack_formats, role)
+
+        self._value = bytearray(0)
+        self._dirty = False
+    
+    @property
+    def value(self) -> bytearray:
+        """
+        Cached reading value
+        """
+        return self._value
+
+    @value.setter
+    def value(self, v: bytearray) -> None:
+        # TODO: check for equality
+        self._value = v or bytearray(0)
+        self._dirty = True
+        # TODO: debounce
+        self.refresh_value()
+
+    @property
+    def dirty(self) -> bool:
+        return self._dirty
+    
+    def set_dirty(self) -> None:
+        self._dirty = True
+    
+    def refresh_value(self) -> None:
+        if self._dirty:
+            self.register(JD_REG_VALUE).set_values(self._value)
+            self._dirty = False
+
+    def update_value_length(self, length: Optional[int]) -> None:
+        l = len(self._value)
+        if (not length is None) and l != length:
+            # harmonize lengths
+            if length > l:
+                self._value = self._value + bytearray(length - l)
+                self._dirty = True
+            else:
+                self._value = self._value[0:length -1]
+                self._dirty = True
+
+
